@@ -300,6 +300,7 @@ dcl(LANG_EN_NZ, 							"English (New Zealand)")
 dcl(LANG_EN_ZA, 							"English (South Africa)")
 dcl(LANG_EN_US, 							"English (US)")
 dcl(LANG_AF_ZA, 							"Afrikaans")
+dcl(LANG_AK_GH, 							"Akan")	// 
 dcl(LANG_SQ_AL, 							"Albanian")	// Jordi 19/10/2002
 dcl(LANG_AM_ET, 							"Amharic (Ethiopia)")
 dcl(LANG_AR_EG, 							"Arabic (Egypt)")
@@ -360,6 +361,7 @@ dcl(LANG_LA_IT, 							"Latin (Renaissance)")	// Is _IT the right thing here?
 dcl(LANG_LV_LV, 							"Latvian")
 dcl(LANG_LT_LT, 							"Lithuanian")
 dcl(LANG_MK,								"Macedonian")	// Jordi 19/10/2002 Hipi: Why not mk-MK?
+dcl(LANG_MS_MY,                             "Malay")
 dcl(LANG_MI_NZ,                             "Maori")
 dcl(LANG_MR_IN,                             "Marathi")
 dcl(LANG_MH_MH, 							"Marshallese (Marshall Islands)")
@@ -392,6 +394,7 @@ dcl(LANG_TH_TH, 							"Thai")
 dcl(LANG_TR_TR, 							"Turkish")
 dcl(LANG_UK_UA, 							"Ukrainian")
 dcl(LANG_UR_PK, 							"Urdu")
+dcl(LANG_UZ_UZ, 							"Uzbek")
 dcl(LANG_VI_VN, 							"Vietnamese")
 dcl(LANG_CY_GB, 							"Welsh")
 dcl(LANG_YI,								"Yiddish")
